<?php

namespace Litecms\Building\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litecms\Building\Events\BuildingWorkflow as BuildingWorkflowEvent;
use Litecms\Building\Notifications\BuildingWorkflow as BuildingWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class BuildingWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(BuildingWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $building is submitted.
     *
     * @param BuildingWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(BuildingWorkflowEvent $event)
    {
        $client = $event->building->client;
        Notification::send($client, new BuildingWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $building is published.
     *
     * @param BuildingWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(BuildingWorkflowEvent $event)
    {
        $client = $event->building->client;
        Notification::send($client, new BuildingWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $building is approved.
     *
     * @param BuildingWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(BuildingWorkflowEvent $event)
    {
        $client = $event->building->client;
        Notification::send($client, new BuildingWorkflowNotification($event));
    }

    /**
     * Handles the $building workflow event as a listener.
     *
     * @param BuildingWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(BuildingWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
